pub const LEVEL_WIDTH: usize = 50;
pub const LEVEL_HEIGHT: usize = 25;

#[derive(Copy, Clone, Debug, PartialEq)]
pub enum LevelElement {
    Wall,
    Floor,
    StairDown,
    StairUp,
}

#[derive(Copy, Clone, Debug, PartialEq)]
pub struct Level {
    structure: [[LevelElement; LEVEL_HEIGHT]; LEVEL_WIDTH],
}

impl Level {
    const ROOM_HEIGHT: usize = 6;
    const ROOM_WIDTH: usize = 7;
    fn draw_room(row: usize, col: usize, s: &mut [[LevelElement; LEVEL_HEIGHT]; LEVEL_WIDTH]) {
        for r in 0..Level::ROOM_HEIGHT {
            for c in 0..Level::ROOM_WIDTH {
                if c == Level::ROOM_WIDTH - 1 || r == Level::ROOM_HEIGHT - 1 {
                    s[col * Level::ROOM_WIDTH + c][row * Level::ROOM_HEIGHT + r] = LevelElement::Wall;
                }
            }
        }
    }
    pub fn new() -> Level {
        let mut s = [[LevelElement::Floor; LEVEL_HEIGHT]; LEVEL_WIDTH];
        Level::draw_room(0, 0, &mut s);
        Level::draw_room(1, 1, &mut s);
        for i in 0..LEVEL_WIDTH {
            s[i][0] = LevelElement::Wall;
        }
        for i in 0..LEVEL_HEIGHT {
            s[0][i] = LevelElement::Wall;
        }

        Level {
            structure: s
        }
    }
    pub fn get_element(&self, x: i16, y: i16) -> Option<LevelElement> {
        if x < 0 || y < 0 {
            return None;
        }
        if x >= LEVEL_WIDTH as i16 || y >= LEVEL_HEIGHT as i16 {
            return None;
        }
        Some(self.structure[x as usize][y as usize])
    }
}

#[test]
fn test_get_element() {
    let l = Level::new();
    assert_eq!(l.get_element(0, 0).unwrap(), LevelElement::Wall);
    assert_eq!(l.get_element(4, 4).unwrap(), LevelElement::StairDown);
    assert_eq!(l.get_element(49, 24).unwrap(), LevelElement::Wall);
    assert_eq!(l.get_element(50, 25), None);
}
